#ifndef MultiPad_h
#define MultiPad_h

#define knobisdown 0
#define knobisreleased 1
#define knobhaschanged 2
#define knobispressed 3
#define knobonce 4

#define knobRoof 200
class MultiPad {
  public:
    MultiPad(int count, int pina, int pinb, int pinc, int pind, int datapin);
    bool down(byte n);
    bool changed(byte n);
    bool pressed(byte n);
    bool released(byte n);
    int  maxpads();
    long getpos(byte n);
    void setpos(byte n, long v);
    void setsens(int v);
    void setbound(int x, int y);
    byte update();
    void callibrate(byte mss);
    void clear();
    int knobsens[16];
  private:
    int pa, pb, pc, pd, data;
    boolean knobits[16][5];
    int knobpos[16];
    int knobposold[16];

    int sample[16][16];
    int sampleindex[16];
    int total[16];

    //int padmin[16];
    //int padmax[16];

    byte samplen, numknobs;
    int minv, maxv;
    int debouncetime;
    int sens = 128;

};

void MultiPad::clear() {
  for (int x = 0; x < numknobs; x++) {
    knobits[x][0] = 0;
    knobits[x][1] = 0;
    knobits[x][2] = 0;
  }
}

void MultiPad::callibrate(byte mss) {
  if (mss < 3) mss = 3;
  for (int padn = 0; padn < numknobs ; padn++) {
    digitalWrite(pa, (padn >> 0 ) & 1);
    digitalWrite(pb, (padn >> 1 ) & 1);
    digitalWrite(pc, (padn >> 2 ) & 1);
    digitalWrite(pd, (padn >> 3 ) & 1);
    long st = millis();
    while (millis() - st < mss) {
      int sensorValue = analogRead(data);
      if (sensorValue > knobsens[padn]) knobsens[padn] = sensorValue + 1;
    }
    //if (sensorValue > padmax[padn]) padmax[padn] = sensorValue;
    //if (sensorValue < padmin[padn]) padmin[padn] = sensorValue;
    //knobsens[padn]++;
  }



}

void MultiPad::setsens(int v) {
  sens = v;
}

int MultiPad::maxpads() {
  return numknobs;
}

void MultiPad::setbound(int x, int y)
{
  minv = x;
  maxv = y;
}

long MultiPad::getpos(byte n)
{
  if (n == 0) return 0;
  return knobpos[n - 1];
  //int val = knobpos[n - 1];
  //if (knobits[n - 1][knobisdown]) return  map(constrain(val,0,knobRoof),0,knobRoof,minv + knobsens[n - 1],maxv);
  return 0;
}

bool MultiPad::pressed(byte n) {
  if (n == 0) return 0;
  //n = n  % numknobs;
  if (knobits[n - 1][knobispressed] && knobits[n - 1][knobonce] == 0) {
    knobits[n - 1][knobonce] = 1;
    knobits[n - 1][knobispressed] = 0;
    return 1;
  }
  return 0;
}

bool MultiPad::released(byte n) {
  if (n == 0) return 0;
  //n = n  % numknobs;
  if (knobits[n - 1][knobisreleased] && knobits[n - 1][knobisdown] == 0) {
    knobits[n - 1][knobisreleased] = 0;
    return 1;
  }
  return 0;
}

bool MultiPad::down(byte n) {
  if (n == 0) return 0;
  //n = n  % numknobs;
  return knobits[n - 1][knobisdown];
}

bool MultiPad::changed(byte n) {
  if (n == 0) return 0;
  //n = n  % numknobs;
  if (knobits[n - 1][knobhaschanged]) {
    knobits[n - 1][knobhaschanged] = 0;
    return 1;
  }
  return 0;
}

byte MultiPad::update(void) {
  byte changed = 0;
  byte samples = 8;
  //readall knobs
  for (int padn = 0; padn < numknobs ; padn++) {
    digitalWrite(pa, (padn >> 0 ) & 1);
    digitalWrite(pb, (padn >> 1 ) & 1);
    digitalWrite(pc, (padn >> 2 ) & 1);
    digitalWrite(pd, (padn >> 3 ) & 1);
    total[padn] = total[padn] - sample[padn][sampleindex[padn]];
    int state = analogRead(data) - knobsens[padn];
    // state = map( state, padmin[padn], padmax[padn], 0, 1023);

    sample[padn][sampleindex[padn]] = state;
    total[padn] = total[padn] + state;
    sampleindex[padn] = sampleindex[padn] + 1;
    sampleindex[padn] = sampleindex[padn] % samples;

    //knobpos[padn] = total[padn] / 8;
    knobpos[padn] =  map(constrain(total[padn] / samples, 0, knobRoof), 0, knobRoof, minv , maxv) ;

    knobits[padn][knobisdown] = knobpos[padn] >= (sens + knobsens[padn]);

    if (knobposold[padn] != knobpos[padn]) {
      knobits[padn][knobisreleased] = (knobits[padn][knobisdown] == 0);//knobpos[padn] <= (sens + knobsens[padn]);
      //if (knobits[padn][knobispressed] = knobits[padn][knobisdown];


      if (knobits[padn][knobisdown] && knobits[padn][knobonce] == 0) {
        knobits[padn][knobispressed] = 1;
      }

      if (knobits[padn][knobisdown] == 0 && knobits[padn][knobonce] == 1) knobits[padn][knobonce] = 0;

      changed++;
      knobits[padn][knobhaschanged] = 1;
      knobposold[padn] = knobpos[padn];
    }



  }
  return changed;
}

MultiPad::MultiPad(int count, int pina, int pinb, int pinc, int pind, int datapin) {
  numknobs = count;
  pa = pina;
  pb = pinb;
  pc = pinc;
  pd = pind;
  data = datapin;
  pinMode(pa, OUTPUT);
  pinMode(pb, OUTPUT);
  pinMode(pc, OUTPUT);
  pinMode(pd, OUTPUT);
  pinMode(data, INPUT);
  long temp = millis();
  minv = 0;
  maxv = 1024;
  //while (millis() - temp < 100){update();}
  //debounce = millis();
  //for (byte x = 0;x < numknobs;x++){
  //moved(x);
  //knobpos[x] = 0;
  //}
}



#endif
